import os
from os.path import expanduser, join
import sys
import json
import getpass

USER_HOME = expanduser("~")


def eprint(text):
    print(text, file=sys.stderr)


class Config:

    def __init__(self, user_home=USER_HOME):
        self.user_home = user_home
        self.config_home = join(self.user_home, '.jenkins-fire-cli')
        self.config_file = join(self.config_home, 'config.json')
        self.jar_libs = join(self.config_home, 'jar_libs')

        os.makedirs(self.config_home, exist_ok=True, mode=0o755)
        os.makedirs(self.jar_libs, exist_ok=True, mode=0o755)

    def set(self, path: str, value):
        config = self._load_config()
        keys = path.split('.')

        next_config = config
        for key in keys[:-1]:
            next_config = next_config.setdefault(key, dict())
        next_config[keys[-1]] = value
        self._write_config(config)

    def get(self, path: str):
        config = self._load_config()
        next_config = config
        for key in path.split('.'):
            if type(next_config) is not dict:
                break
            next_config = next_config.get(key, None)
        return next_config

    def show(self):
        print(self._read_config())

    @property
    def username(self):
        return os.environ.get('JENKINS_USER_ID', self.get('user.name'))

    @property
    def token(self):
        return os.environ.get('JENKINS_API_TOKEN', self.get('user.token')) or getpass.getpass()

    @property
    def job_dsl_core_path(self):
        return os.environ.get('JENKINS_JOB_DSL_PATH', join(self.jar_libs, 'job-dsl-core-standalone.jar'))

    @property
    def jenkins_url(self):
        url = os.environ.get('JENKINS_URL', self.get('jenkins.url')) or 'http://localhost:8080'
        return url.rstrip('/')

    @property
    def jenkins_cli_download_url(self):
        return '{}/jnlpJars/jenkins-cli.jar'.format(self.jenkins_url)

    @property
    def jenkins_cli_path(self):
        return os.environ.get('JENKINS_CLI_PATH', join(self.jar_libs, 'jenkins-cli.jar'))

    def _read_config(self):
        try:
            with open(self.config_file, 'r', encoding='utf-8') as fp:
                return fp.read()
        except FileNotFoundError:
            return ''

    def _load_config(self):
        s = self._read_config()
        if not s:
            return dict()
        try:
            return json.loads(s)
        except json.JSONDecodeError as e:
            eprint('Fail to parse configuration, please check {}'.format(
                self.config_file))
            raise e

    def _write_config(self, config):
        with open(self.config_file, 'w') as fp:
            json.dump(config, fp, sort_keys=True, indent=2)
        # use file mode 600 as token is save in it
        os.chmod(self.config_file, 0o600)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jenkins_fire_cli']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['jenkins-fire-cli = jenkins_fire_cli.main:main']}

setup_kwargs = {
    'name': 'jenkins-fire-cli',
    'version': '0.1.1',
    'description': '',
    'long_description': '# jenkins-fire-cli\nA Jenkins command line tool built with python-fire that wraps the jenkins-cli and job-dsl-core.\n\n## Prerequisite \nYou need to ensure `Java` and `Python>=3.8` are in the PATH to use this tool. \n\n## Install\nIt can be installed by the following command.\n```bash\npip install jenkins-fire-cli\n```\n\n## Get Started\n\n### Provision\nYou need to set Jenkins site url and user credential for the first time\n\n```bash\njenkins-fire-cli config set jenkins.url http://your-jenkins-site\njenkins-fire-cli config set user.name john.doe\n\n# user.token is optional, but you will be asked to type it when it is missing\n# either api token or user\'s password will be OK, but it\'s recommneded to use token for the sake of security\njenkins-fire-cli config set user.token passw0rd \n```\n\nBefore you start to run Jenkins commands, don\'t forget to run the `init` command for the first time, which will download `jenkins-cli.jar` and `job-dsl-core-standalone.jar` automatically.\n\n```bash\njenkins-fire-cli init\n```\n\n### Run jenkins-cli commnads\nAs `jenkins-fire-cli` is a wrapper of `jenkins-cli` and `job-dsl-core` to make them easier to use, \nyou can find the document of `jenkins-cli` in your Jenkins site or run the following command to open in browser.\n\n```bash\njenkins-fire-cli doc --open\n```\n\nYou can use the `run` sub-command to execute `jenkins-cli` command, for example\n\n```bash\njenkins-fire-cli run list-jobs  \n# It is equivalent to "java -jar jenkins-cli.jar -s http://localhost:9090/ -webSocket list-jobs"\n\n# For command with multiple arguments you need to quote them with ""\njenkins-fire-cli run "get-job my-job"  \n```\n\n### Run job-dsl commands\n\nYou can also run the `job-dsl` command. For example you have a `job-dsl` script with the following content.\n\n```groovy\n// file: /tmp/sample.dsl\npipelineJob(\'job-dsl-plugin\') {\n  definition {\n    cpsScm {\n      scm {\n        git {\n          remote {\n            url(\'https://github.com/jenkinsci/job-dsl-plugin.git\')\n          }\n          branch(\'*/master\')\n        }\n      }\n      lightweight()\n    }\n  }\n}\n```\n\nThen you can run the following command to generate the job XML configuration.\n\n```bash\njenkins-fire-cli dsl /tmp/sample-dsl.groovy\n```\n\nThen you will find a file named `job-dsl-plugin.xml` is generated. \nNow you can run anthor command to create this job in jenkins:\n\n```bash\njenkins-file-cli run \'create-job job-dsl-plugin\' < job-dsl-plugin.xml\n```\n\nThen you will find a new job named `job-dsl-plugin` has been created in jenkins.\n\n### Environment Variables\n\nIf you want to use this tool in CI system, you may use the following environment variable instead of global setting.\n\n* `JENKINS_USER_ID`: equivalent to `user.name`\n* `JENKINS_API_TOKEN`: equvalent to `user.token`\n* `JENKINS_URL`: equvalent to `jenkins.url`\n* `JENKINS_JOB_DSL_PATH`: path to the job-dsl jar package, you may skip `init` when this is set \n* `JNEKINS_CLI_PATH`: path to the jenkins-cli jar package, you may skip `init` when this is set \n',
    'author': 'weihong.xu',
    'author_email': 'xuweihong.cn@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from .config import Config
import fire
from urllib.request import urlretrieve, urlopen
import os
import os.path
import xml.etree.ElementTree as ET
import subprocess as sp
import shlex
import webbrowser


class Entry:

    def __init__(self, config: Config):
        self.config = config

    @property
    def jenkins_cli_base_cmd(self):
        return ["java", "-jar", self.config.jenkins_cli_path, "-s", self.config.jenkins_url,  "-webSocket"]
    
    @property
    def job_dsl_base_cmd(self):
        return ["java", "-jar", self.config.job_dsl_core_path, '-j']

    @property
    def jenkins_env(self):
        env = os.environ.copy()
        env['JENKINS_USER_ID'] = self.config.username
        env['JENKINS_API_TOKEN'] = self.config.token
        return env

    # TODO: currently fire doesn't support pass raw arguments, so the command should pass as string
    # ref: https://github.com/google/python-fire/issues/403
    def run(self, cmd: str):
        sp.call(self.jenkins_cli_base_cmd +
                shlex.split(cmd), env=self.jenkins_env)
    
    def doc(self, open=False):
        url = "{}/manage/cli/".format(self.config.jenkins_url)
        print(url)
        if open:
            webbrowser.open(url)

    def dsl(self, file: str):
        sp.call(self.job_dsl_base_cmd + [file])

    def init(self, jenkins_cli_url: str = None, jenkins_job_dsl_core_url: str = None, force_download=False):
        self._download_job_dsl_core(jenkins_job_dsl_core_url, force_download)
        self._download_jenkins_cli(jenkins_cli_url, force_download)
    
    def _get_job_dsl_core_release_version(self):
        url = 'https://repo.jenkins-ci.org/public/org/jenkins-ci/plugins/job-dsl-core/maven-metadata.xml'
        with urlopen(url) as fp:
            tree = ET.parse(fp)
        return tree.find('./versioning/release').text

    def _download_job_dsl_core(self, url: str = None, force_download=False):
        version = self._get_job_dsl_core_release_version()
        url = url or 'https://repo.jenkins-ci.org/public/org/jenkins-ci/plugins/job-dsl-core/{version}/job-dsl-core-{version}-standalone.jar'.format(
            version=version)
        target = self.config.job_dsl_core_path
        self._download_file(url, target, force_download)

    def _download_jenkins_cli(self, url: str = None, force_download=False):
        url = url or self.config.jenkins_cli_download_url
        target = self.config.jenkins_cli_path
        self._download_file(url, target, force_download)

    def _download_file(self, url: str, target: str, force_download: bool):
        if not force_download and os.path.exists(target):
            return
        print('Download {} to {} ...'.format(url, target))
        urlretrieve(url, target)


def main():
    config = Config()
    entry = Entry(config)
    fire.Fire(entry)

    
if __name__ == '__main__':
    main()
